# Domoticz Python Plugin alternative to the Logitech Harmony hardware 
# Requires https://github.com/bwssytems/restful-harmony
#
# Author: ESCape
#
"""
<plugin key="restharmony" name="Logitech H-rmony restful plugin" author="ESCape" version="0.0.4">
	<params>
		<param field="Address" label="Restful-harmony Address" width="200px" required="true" default="localhost"/>
		<param field="Port" label="Restful-harmony port" width="200px" required="true" default="8081"/>
		<param field="Mode1" label="Check current activity every" width="200px" required="true">
			<options>
				<option label="5 seconds" value="5"/>
				<option label="10 seconds" value="10" default="true"/>
				<option label="1 minute" value="60"/>
			</options>
		</param>
		<param field="Mode2" label="Connection timeout when polling status" width="200px" required="true">
			<options>
				<option label="1 second (preffered)" value="1" default="true"/>
				<option label="2 seconds" value="2"/>
				<option label="3 secons (might block event system when interval is short)" value="3"/>
			</options>
		</param>		
		<param field="Mode3" label="Number of retries before logging a failure" width="200px" required="true">
			<options>
				<option label="Log (connection) failure immediately" value="0"/>
				<option label="5" value="5" default="true"/>
				<option label="10" value="10"/>
			</options>
		</param>		
	</params>
</plugin>
"""

import Domoticz
import urllib.request
import json

class BasePlugin:
	def __init__(self):
		#self.var = 123
		self.failures=0
		return
	
	#map Harmony (external) activity ID's to internal Domoticz ID's
	def act2id(self):
		actlist={}
		for thing in Devices:
			actlist[Devices[thing].DeviceID]=thing
		return actlist
	
	#find the first available (internal) unit id
	def firstfree(self):
		for num in range(1,250):
			if num not in Devices:
				return num
		return
	
	#Get all activities from Harmony hub and convert Harmony activity ID to Hexadecimal format
	def activities(self, apihost='localhost', apiport='8081'):
		Domoticz.Log('Gettings Harmony activities from: ' + 'http://'+apihost+':'+apiport+'/harmony/list/activities')
		try:
			#data = requests.get('http://'+apihost+':'+apiport+'/harmony/list/activities', timeout=1)
			webdata = urllib.request.urlopen('http://'+apihost+':'+apiport+'/harmony/list/activities', timeout=3)
		except:
			Domoticz.Error("Could not connect to Harmony Hub.")
			return
		if webdata.getcode() != 200:
			Domoticz.Error("Could not get valid data from Harmony Hub. Error:" + webdata.getcode())
			return
		else:
			raw=webdata.read()
			encoding = webdata.info().get_content_charset('utf-8')
			parsed=json.loads(raw.decode(encoding))
			myacts = {}
			for act in parsed:
				if act["id"]==-1:
					#results in FFFFFFFF hex
					useid=format(4294967295, 'X')
				else:
					useid=format(int(act["id"]), 'X')
				myacts[useid]= act["label"]
			return myacts
		return

	#Get current activity from Harmony hub and convert Harmony activity ID to Hexadecimal format
	def currentactivity(self, apihost='localhost', apiport='8081'):
		try:
			webdata = urllib.request.urlopen('http://'+apihost+':'+apiport+'/harmony/show/activity', timeout=self.polltimeout)
		except:
			if self.failures >= self.retries:
				Domoticz.Error("Could not connect to Harmony Hub.")
				self.failures=0
			else:
				self.failures += 1
			return
		if webdata.getcode() != 200:
			if self.failures >= self.retries:
				Domoticz.Error("Could not get valid data from Harmony Hub. Resonse:" + webdata.getcode())
				self.failures=0
			else:
				self.failures += 1
			return
		else:
			self.failures=0
			raw=webdata.read()
			encoding = webdata.info().get_content_charset('utf-8')
			parsed=json.loads(raw.decode(encoding))
			if parsed["id"]==-1:
				curact=format(4294967295, 'X')
			else:
				curact=format(int(parsed["id"]), 'X')
			return curact
		return
			
	def setactivitystatus(self, activity):
		Devices[self.harmonyidlist[activity]].Update(nValue=1, sValue="On")
		for other in Devices:
			if other != self.harmonyidlist[activity]:
				if Devices[other].nValue == 1:
					Devices[other].Update(nValue=0, sValue="Off")
		return
				
	def sendharmony(self, activity, apihost='localhost', apiport='8081'):
		Domoticz.Log('Send2harmony activity:'+str(activity))
		if activity == 4294967295:
			realactivity = -1
		else:
			realactivity = activity
		url='http://' + apihost + ':' + apiport + '/harmony/start'
		command = {"activityid":realactivity} 
		params = json.dumps(command).encode('utf8')
		req = urllib.request.Request(url, method='PUT', data=params, headers={"Content-Type": "application/json"})
		try:
			response = urllib.request.urlopen(req, timeout=3)
		except:
			Domoticz.Error('Failed to send Harmony command (connection failure?)')
			return False
		if response.getcode() != 200:
			Domoticz.Error('Unexpected response code from restful Harmony: ' + str(response.getcode()))
			return False
		return True

	def onStart(self):
		self.restaddress=Parameters["Address"]
		self.restport=Parameters["Port"]
		self.polltimeout=int(Parameters["Mode2"])
		self.retries=int(Parameters["Mode3"])
		Domoticz.Heartbeat(int(Parameters["Mode1"]))
		allactivities=self.activities(apihost=self.restaddress, apiport=self.restport)

		#Select or create icons for devices 
		icon="Restharmony"
		if icon not in Images: Domoticz.Image('restharmony_icons.zip').Create()
		iconid=Images[icon].ID
		
		#add new activities
		self.harmonyidlist=self.act2id()
		if allactivities is not None:
			for actid in allactivities:
				if actid not in self.harmonyidlist:
					Domoticz.Device(Name=allactivities[actid], Unit=self.firstfree(), DeviceID=actid, TypeName="Switch", Used=1, Image=iconid).Create()
					Domoticz.Log("Harmony activity " + allactivities[actid] + " added.")
			self.harmonyidlist=self.act2id()
		
			#find and remove obsolete activities
			obsolete=[]
			for check in Devices:
				if Devices[check].DeviceID not in allactivities:
					obsolete.append(check)
			for trash in obsolete:
				Domoticz.Log("Removing Harmony activity " + str(Devices[trash].Name) + " (obsolete).")
				Devices[trash].Delete()
			self.harmonyidlist=self.act2id()
		else:
			Domoticz.Error("Unable to get Harmony activities. Check if restful Harmony is running before starting this plugin.")
		return

	def onStop(self):
		Domoticz.Log("onStop called")

	def onConnect(self, Connection, Status, Description):
		Domoticz.Log("onConnect called")

	def onMessage(self, Connection, Data):
		Domoticz.Log("onMessage called")

	def onCommand(self, Unit, Command, Level, Hue):
		if str(Command)=='On':
			harmonyact=int(Devices[Unit].DeviceID, 16)	
			if self.sendharmony(harmonyact, apihost=self.restaddress, apiport=self.restport):
				Domoticz.Log('Harmony Activity set to ' + str(Devices[Unit].DeviceID) + '=>' + str(harmonyact))
				self.setactivitystatus(Devices[Unit].DeviceID)
			else:
				Domoticz.Error('Failed to set activity')
		return

	def onNotification(self, Name, Subject, Text, Status, Priority, Sound, ImageFile):
		Domoticz.Log("Notification: " + Name + "," + Subject + "," + Text + "," + Status + "," + str(Priority) + "," + Sound + "," + ImageFile)

	def onDisconnect(self, Connection):
		Domoticz.Log("onDisconnect called")

	def onHeartbeat(self):
		thisact=self.currentactivity(apihost=self.restaddress, apiport=self.restport)
		if thisact is not None:
			if Devices[self.harmonyidlist[thisact]].nValue != 1:
				self.setactivitystatus(thisact)
		else:
			Domoticz.Debug("Could not retreive current activity")
		return

global _plugin
_plugin = BasePlugin()

def onStart():
	global _plugin
	_plugin.onStart()

def onStop():
	global _plugin
	_plugin.onStop()

def onConnect(Connection, Status, Description):
	global _plugin
	_plugin.onConnect(Connection, Status, Description)

def onMessage(Connection, Data):
	global _plugin
	_plugin.onMessage(Connection, Data)

def onCommand(Unit, Command, Level, Hue):
	global _plugin
	_plugin.onCommand(Unit, Command, Level, Hue)

def onNotification(Name, Subject, Text, Status, Priority, Sound, ImageFile):
	global _plugin
	_plugin.onNotification(Name, Subject, Text, Status, Priority, Sound, ImageFile)

def onDisconnect(Connection):
	global _plugin
	_plugin.onDisconnect(Connection)

def onHeartbeat():
	global _plugin
	_plugin.onHeartbeat()

	# Generic helper functions
def DumpConfigToLog():
	for x in Parameters:
		if Parameters[x] != "":
			Domoticz.Debug( "'" + x + "':'" + str(Parameters[x]) + "'")
	Domoticz.Debug("Device count: " + str(len(Devices)))
	for x in Devices:
		Domoticz.Debug("Device:			  " + str(x) + " - " + str(Devices[x]))
		Domoticz.Debug("Device ID:		 '" + str(Devices[x].ID) + "'")
		Domoticz.Debug("Device Name:	 '" + Devices[x].Name + "'")
		Domoticz.Debug("Device nValue:	  " + str(Devices[x].nValue))
		Domoticz.Debug("Device sValue:	 '" + Devices[x].sValue + "'")
		Domoticz.Debug("Device LastLevel: " + str(Devices[x].LastLevel))
	return
